import obs, uasyncio, config
from modbus import modbus
from sensor_drv import SensorDrv

class modbus_opus(SensorDrv):
    def __init__(self, serial, conf):
        SensorDrv.__init__(self)
        self.__syslog = obs.system_log('modbus_opus')
        self.modbus_obj = modbus(serial, conf)
        self.__serial = serial
        self.wait_time = config.get_config_or_default(conf, 'measurement_wait_time', 25) * 1000
        self.retry_nr = 5

    async def read_values(self):
        #Start by sending a measurment request, then wait the 'measurement_wait_time' that is defined in config 
        await self.measurement_request()
        #read register as usual, retry up to five times. If no response the driver managers initiates this function again up to the response_timeout.
        for _ in range(self.retry_nr):
            if await self.modbus_obj.read_values() == True:
                self.data = self.modbus_obj.data
                return True
            await uasyncio.sleep_ms(1000)
        return False

    async def detect_parameters(self, tag_number = ''):
        await self.measurement_request()
        for _ in range(self.retry_nr):
            if await self.modbus_obj.detect_parameters(tag_number):
                self.parameters = self.modbus_obj.parameters
                return True
            await uasyncio.sleep_ms(1000)
        return False

    async def measurement_request(self):
        #Write register to start the measurement
        self.__serial.write(b'\x01\x06\x00\x01\x01\x01\x18\x5A')
        self.__syslog.info('Transmitted the measurement command, wait time: {} sec.'.format(self.wait_time/1000))
        #Wait for self.wait_time ms for the sensor to obtain values
        await uasyncio.sleep_ms(self.wait_time)

    async def flush(self):
        #Not required
        pass
    

